@@ -75,13 +75,13 @@ func (s *VisibilityPersistenceSuite) TestBasicVisibility() {
 		RunId:      "fb15e4b5-356f-466d-8c6d-a29223e5c536",
 	}
 
-	startTime := time.Now().UTC().Add(time.Second * -5).UnixNano()
+	startTime := time.Now().UTC().Add(time.Second * -5)
 	startReq := &p.RecordWorkflowExecutionStartedRequest{
 		VisibilityRequestBase: &p.VisibilityRequestBase{
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
 	}
 	err0 := s.VisibilityMgr.RecordWorkflowExecutionStarted(startReq)
@@ -102,10 +102,10 @@ func (s *VisibilityPersistenceSuite) TestBasicVisibility() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
-		CloseTimestamp: time.Now().UnixNano(),
-		HistoryLength:  5,
+		CloseTime:     time.Now(),
+		HistoryLength: 5,
 	}
 	err2 := s.VisibilityMgr.RecordWorkflowExecutionClosed(closeReq)
 	s.Nil(err2)
@@ -123,7 +123,7 @@ func (s *VisibilityPersistenceSuite) TestBasicVisibility() {
 		NamespaceID:       testNamespaceUUID,
 		PageSize:          1,
 		EarliestStartTime: startTime,
-		LatestStartTime:   time.Now().UnixNano(),
+		LatestStartTime:   time.Now(),
 	})
 	s.Nil(err4)
 	s.Equal(1, len(resp.Executions))
@@ -139,13 +139,13 @@ func (s *VisibilityPersistenceSuite) TestBasicVisibilityTimeSkew() {
 		RunId:      "fb15e4b5-356f-466d-8c6d-a29223e5c536",
 	}
 
-	startTime := time.Now().UTC().UnixNano()
+	startTime := time.Now().UTC()
 	err0 := s.VisibilityMgr.RecordWorkflowExecutionStarted(&p.RecordWorkflowExecutionStartedRequest{
 		VisibilityRequestBase: &p.VisibilityRequestBase{
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
 	})
 	s.NoError(err0)
@@ -165,9 +165,9 @@ func (s *VisibilityPersistenceSuite) TestBasicVisibilityTimeSkew() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
-		CloseTimestamp: startTime - (10 * time.Millisecond).Nanoseconds(),
+		CloseTime: startTime.Add(-10 * time.Millisecond),
 	})
 	s.NoError(err2)
 
@@ -183,8 +183,8 @@ func (s *VisibilityPersistenceSuite) TestBasicVisibilityTimeSkew() {
 	resp, err4 := s.VisibilityMgr.ListClosedWorkflowExecutions(&p.ListWorkflowExecutionsRequest{
 		NamespaceID:       testNamespaceUUID,
 		PageSize:          1,
-		EarliestStartTime: startTime - (10 * time.Millisecond).Nanoseconds(), // This is actually close_time
-		LatestStartTime:   startTime - (10 * time.Millisecond).Nanoseconds(),
+		EarliestStartTime: startTime.Add(-10 * time.Millisecond), // This is actually close_time
+		LatestStartTime:   startTime.Add(-10 * time.Millisecond),
 	})
 	s.NoError(err4)
 	s.Equal(1, len(resp.Executions))
@@ -198,13 +198,13 @@ func (s *VisibilityPersistenceSuite) TestBasicVisibilityShortWorkflow() {
 		RunId:      "3c095198-0c33-4136-939a-c29fbbb6a80b",
 	}
 
-	startTime := time.Now().UTC().UnixNano()
+	startTime := time.Now().UTC()
 	err0 := s.VisibilityMgr.RecordWorkflowExecutionStarted(&p.RecordWorkflowExecutionStartedRequest{
 		VisibilityRequestBase: &p.VisibilityRequestBase{
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
 	})
 	s.NoError(err0)
@@ -214,9 +214,9 @@ func (s *VisibilityPersistenceSuite) TestBasicVisibilityShortWorkflow() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
-		CloseTimestamp: startTime + (10 * time.Millisecond).Nanoseconds(),
+		CloseTime: startTime.Add(10 * time.Millisecond),
 	})
 	s.NoError(err2)
 
@@ -232,8 +232,8 @@ func (s *VisibilityPersistenceSuite) TestBasicVisibilityShortWorkflow() {
 	resp, err4 := s.VisibilityMgr.ListClosedWorkflowExecutions(&p.ListWorkflowExecutionsRequest{
 		NamespaceID:       testNamespaceUUID,
 		PageSize:          1,
-		EarliestStartTime: startTime + (10 * time.Millisecond).Nanoseconds(), // This is actually close_time
-		LatestStartTime:   startTime + (10 * time.Millisecond).Nanoseconds(),
+		EarliestStartTime: startTime.Add(10 * time.Millisecond), // This is actually close_time
+		LatestStartTime:   startTime.Add(10 * time.Millisecond),
 	})
 	s.NoError(err4)
 	s.Equal(1, len(resp.Executions))
@@ -251,13 +251,13 @@ func (s *VisibilityPersistenceSuite) TestVisibilityRetention() {
 		RunId:      "3c095198-0c33-4136-939a-c29fbbb6a802",
 	}
 
-	startTime := time.Now().UTC().Add(-1 * time.Hour).UnixNano()
+	startTime := time.Now().UTC().Add(-1 * time.Hour)
 	err0 := s.VisibilityMgr.RecordWorkflowExecutionStarted(&p.RecordWorkflowExecutionStartedRequest{
 		VisibilityRequestBase: &p.VisibilityRequestBase{
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
 	})
 	s.NoError(err0)
@@ -268,10 +268,10 @@ func (s *VisibilityPersistenceSuite) TestVisibilityRetention() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
-		CloseTimestamp: startTime + (1 * time.Minute).Nanoseconds(),
-		Retention:      &retention,
+		CloseTime: startTime.Add(1 * time.Minute),
+		Retention: &retention,
 	})
 	s.NoError(err2)
 
@@ -287,8 +287,8 @@ func (s *VisibilityPersistenceSuite) TestVisibilityRetention() {
 	resp, err4 := s.VisibilityMgr.ListClosedWorkflowExecutions(&p.ListWorkflowExecutionsRequest{
 		NamespaceID:       testNamespaceUUID,
 		PageSize:          1,
-		EarliestStartTime: startTime + (1 * time.Minute).Nanoseconds(), // This is actually close_time
-		LatestStartTime:   startTime + (1 * time.Minute).Nanoseconds(),
+		EarliestStartTime: startTime.Add(1 * time.Minute), // This is actually close_time
+		LatestStartTime:   startTime.Add(1 * time.Minute),
 	})
 	s.NoError(err4)
 	s.Equal(1, len(resp.Executions))
@@ -298,8 +298,8 @@ func (s *VisibilityPersistenceSuite) TestVisibilityRetention() {
 	resp2, err5 := s.VisibilityMgr.ListClosedWorkflowExecutions(&p.ListWorkflowExecutionsRequest{
 		NamespaceID:       testNamespaceUUID,
 		PageSize:          1,
-		EarliestStartTime: startTime + (1 * time.Minute).Nanoseconds(), // This is actually close_time
-		LatestStartTime:   startTime + (1 * time.Minute).Nanoseconds(),
+		EarliestStartTime: startTime.Add(1 * time.Minute), // This is actually close_time
+		LatestStartTime:   startTime.Add(1 * time.Minute),
 	})
 	s.NoError(err5)
 	s.Equal(0, len(resp2.Executions))
@@ -321,7 +321,7 @@ func (s *VisibilityPersistenceSuite) TestVisibilityPagination() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution1,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime1.UnixNano(),
+			StartTime:        startTime1,
 		},
 	}
 
@@ -339,7 +339,7 @@ func (s *VisibilityPersistenceSuite) TestVisibilityPagination() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution2,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime2.UnixNano(),
+			StartTime:        startTime2,
 		},
 	}
 	err1 := s.VisibilityMgr.RecordWorkflowExecutionStarted(startReq2)
@@ -349,8 +349,8 @@ func (s *VisibilityPersistenceSuite) TestVisibilityPagination() {
 	resp, err2 := s.VisibilityMgr.ListOpenWorkflowExecutions(&p.ListWorkflowExecutionsRequest{
 		NamespaceID:       testNamespaceUUID,
 		PageSize:          1,
-		EarliestStartTime: startTime1.UnixNano(),
-		LatestStartTime:   startTime2.UnixNano(),
+		EarliestStartTime: startTime1,
+		LatestStartTime:   startTime2,
 	})
 	s.Nil(err2)
 	s.Equal(1, len(resp.Executions))
@@ -360,8 +360,8 @@ func (s *VisibilityPersistenceSuite) TestVisibilityPagination() {
 	resp, err3 := s.VisibilityMgr.ListOpenWorkflowExecutions(&p.ListWorkflowExecutionsRequest{
 		NamespaceID:       testNamespaceUUID,
 		PageSize:          1,
-		EarliestStartTime: startTime1.UnixNano(),
-		LatestStartTime:   startTime2.UnixNano(),
+		EarliestStartTime: startTime1,
+		LatestStartTime:   startTime2,
 		NextPageToken:     resp.NextPageToken,
 	})
 	s.Nil(err3)
@@ -374,8 +374,8 @@ func (s *VisibilityPersistenceSuite) TestVisibilityPagination() {
 		resp, err4 := s.VisibilityMgr.ListOpenWorkflowExecutions(&p.ListWorkflowExecutionsRequest{
 			NamespaceID:       testNamespaceUUID,
 			PageSize:          1,
-			EarliestStartTime: startTime1.UnixNano(),
-			LatestStartTime:   startTime2.UnixNano(),
+			EarliestStartTime: startTime1,
+			LatestStartTime:   startTime2,
 			NextPageToken:     resp.NextPageToken,
 		})
 		s.Nil(err4)
@@ -386,7 +386,7 @@ func (s *VisibilityPersistenceSuite) TestVisibilityPagination() {
 // TestFilteringByType test
 func (s *VisibilityPersistenceSuite) TestFilteringByType() {
 	testNamespaceUUID := uuid.New()
-	startTime := time.Now().UnixNano()
+	startTime := time.Now()
 
 	// Create 2 executions
 	workflowExecution1 := commonpb.WorkflowExecution{
@@ -398,7 +398,7 @@ func (s *VisibilityPersistenceSuite) TestFilteringByType() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution1,
 			WorkflowTypeName: "visibility-workflow-1",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
 	})
 	s.Nil(err0)
@@ -412,7 +412,7 @@ func (s *VisibilityPersistenceSuite) TestFilteringByType() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution2,
 			WorkflowTypeName: "visibility-workflow-2",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
 	})
 	s.Nil(err1)
@@ -437,9 +437,9 @@ func (s *VisibilityPersistenceSuite) TestFilteringByType() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution1,
 			WorkflowTypeName: "visibility-workflow-1",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
-		CloseTimestamp: time.Now().UnixNano(),
+		CloseTime: time.Now(),
 	})
 	s.Nil(err3)
 
@@ -448,10 +448,10 @@ func (s *VisibilityPersistenceSuite) TestFilteringByType() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution2,
 			WorkflowTypeName: "visibility-workflow-2",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
-		CloseTimestamp: time.Now().UnixNano(),
-		HistoryLength:  3,
+		CloseTime:     time.Now(),
+		HistoryLength: 3,
 	}
 	err4 := s.VisibilityMgr.RecordWorkflowExecutionClosed(closeReq)
 	s.Nil(err4)
@@ -462,7 +462,7 @@ func (s *VisibilityPersistenceSuite) TestFilteringByType() {
 			NamespaceID:       testNamespaceUUID,
 			PageSize:          2,
 			EarliestStartTime: startTime,
-			LatestStartTime:   time.Now().UnixNano(),
+			LatestStartTime:   time.Now(),
 		},
 		WorkflowTypeName: "visibility-workflow-2",
 	})
@@ -474,7 +474,7 @@ func (s *VisibilityPersistenceSuite) TestFilteringByType() {
 // TestFilteringByWorkflowID test
 func (s *VisibilityPersistenceSuite) TestFilteringByWorkflowID() {
 	testNamespaceUUID := uuid.New()
-	startTime := time.Now().UnixNano()
+	startTime := time.Now()
 
 	// Create 2 executions
 	workflowExecution1 := commonpb.WorkflowExecution{
@@ -486,7 +486,7 @@ func (s *VisibilityPersistenceSuite) TestFilteringByWorkflowID() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution1,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
 	})
 	s.Nil(err0)
@@ -500,7 +500,7 @@ func (s *VisibilityPersistenceSuite) TestFilteringByWorkflowID() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution2,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
 	})
 	s.Nil(err1)
@@ -525,9 +525,9 @@ func (s *VisibilityPersistenceSuite) TestFilteringByWorkflowID() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution1,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
-		CloseTimestamp: time.Now().UnixNano(),
+		CloseTime: time.Now(),
 	})
 	s.Nil(err3)
 
@@ -536,10 +536,10 @@ func (s *VisibilityPersistenceSuite) TestFilteringByWorkflowID() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution2,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
-		CloseTimestamp: time.Now().UnixNano(),
-		HistoryLength:  3,
+		CloseTime:     time.Now(),
+		HistoryLength: 3,
 	}
 	err4 := s.VisibilityMgr.RecordWorkflowExecutionClosed(closeReq)
 	s.Nil(err4)
@@ -550,7 +550,7 @@ func (s *VisibilityPersistenceSuite) TestFilteringByWorkflowID() {
 			NamespaceID:       testNamespaceUUID,
 			PageSize:          2,
 			EarliestStartTime: startTime,
-			LatestStartTime:   time.Now().UnixNano(),
+			LatestStartTime:   time.Now(),
 		},
 		WorkflowID: "visibility-filtering-test2",
 	})
@@ -562,7 +562,7 @@ func (s *VisibilityPersistenceSuite) TestFilteringByWorkflowID() {
 // TestFilteringByStatus test
 func (s *VisibilityPersistenceSuite) TestFilteringByStatus() {
 	testNamespaceUUID := uuid.New()
-	startTime := time.Now().UnixNano()
+	startTime := time.Now()
 
 	// Create 2 executions
 	workflowExecution1 := commonpb.WorkflowExecution{
@@ -574,7 +574,7 @@ func (s *VisibilityPersistenceSuite) TestFilteringByStatus() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution1,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
 	})
 	s.Nil(err0)
@@ -588,7 +588,7 @@ func (s *VisibilityPersistenceSuite) TestFilteringByStatus() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution2,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
 	})
 	s.Nil(err1)
@@ -599,10 +599,10 @@ func (s *VisibilityPersistenceSuite) TestFilteringByStatus() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution1,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 			Status:           enumspb.WORKFLOW_EXECUTION_STATUS_COMPLETED,
 		},
-		CloseTimestamp: time.Now().UnixNano(),
+		CloseTime: time.Now(),
 	})
 	s.Nil(err2)
 
@@ -611,11 +611,11 @@ func (s *VisibilityPersistenceSuite) TestFilteringByStatus() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution2,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 			Status:           enumspb.WORKFLOW_EXECUTION_STATUS_FAILED,
 		},
-		CloseTimestamp: time.Now().UnixNano(),
-		HistoryLength:  3,
+		CloseTime:     time.Now(),
+		HistoryLength: 3,
 	}
 	err3 := s.VisibilityMgr.RecordWorkflowExecutionClosed(closeReq)
 	s.Nil(err3)
@@ -626,7 +626,7 @@ func (s *VisibilityPersistenceSuite) TestFilteringByStatus() {
 			NamespaceID:       testNamespaceUUID,
 			PageSize:          2,
 			EarliestStartTime: startTime,
-			LatestStartTime:   time.Now().UnixNano(),
+			LatestStartTime:   time.Now(),
 		},
 		Status: enumspb.WORKFLOW_EXECUTION_STATUS_FAILED,
 	})
@@ -644,13 +644,13 @@ func (s *VisibilityPersistenceSuite) TestGetClosedExecution() {
 		RunId:      "a3dbc7bf-deb1-4946-b57c-cf0615ea553f",
 	}
 
-	startTime := time.Now().UTC().Add(time.Second * -5).UnixNano()
+	startTime := time.Now().UTC().Add(time.Second * -5)
 	err0 := s.VisibilityMgr.RecordWorkflowExecutionStarted(&p.RecordWorkflowExecutionStartedRequest{
 		VisibilityRequestBase: &p.VisibilityRequestBase{
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 		},
 	})
 	s.Nil(err0)
@@ -669,11 +669,11 @@ func (s *VisibilityPersistenceSuite) TestGetClosedExecution() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 			Status:           enumspb.WORKFLOW_EXECUTION_STATUS_FAILED,
 		},
-		CloseTimestamp: time.Now().UnixNano(),
-		HistoryLength:  3,
+		CloseTime:     time.Now(),
+		HistoryLength: 3,
 	}
 	err2 := s.VisibilityMgr.RecordWorkflowExecutionClosed(closeReq)
 	s.Nil(err2)
@@ -708,11 +708,11 @@ func (s *VisibilityPersistenceSuite) TestClosedWithoutStarted() {
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   time.Now().UTC().Add(time.Second * -5).UnixNano(),
+			StartTime:        time.Now().UTC().Add(time.Second * -5),
 			Status:           enumspb.WORKFLOW_EXECUTION_STATUS_FAILED,
 		},
-		CloseTimestamp: time.Now().UnixNano(),
-		HistoryLength:  3,
+		CloseTime:     time.Now(),
+		HistoryLength: 3,
 	}
 	err1 := s.VisibilityMgr.RecordWorkflowExecutionClosed(closeReq)
 	s.Nil(err1)
@@ -734,17 +734,17 @@ func (s *VisibilityPersistenceSuite) TestMultipleUpserts() {
 		RunId:      "a3dbc7bf-deb1-4946-b57c-cf0615ea553f",
 	}
 
-	startTime := time.Now().UTC().Add(time.Second * -5).UnixNano()
+	startTime := time.Now().UTC().Add(time.Second * -5)
 	closeReq := &p.RecordWorkflowExecutionClosedRequest{
 		VisibilityRequestBase: &p.VisibilityRequestBase{
 			NamespaceID:      testNamespaceUUID,
 			Execution:        workflowExecution,
 			WorkflowTypeName: "visibility-workflow",
-			StartTimestamp:   startTime,
+			StartTime:        startTime,
 			Status:           enumspb.WORKFLOW_EXECUTION_STATUS_FAILED,
 		},
-		CloseTimestamp: time.Now().UnixNano(),
-		HistoryLength:  3,
+		CloseTime:     time.Now(),
+		HistoryLength: 3,
 	}
 
 	count := 3
@@ -754,7 +754,7 @@ func (s *VisibilityPersistenceSuite) TestMultipleUpserts() {
 				NamespaceID:      testNamespaceUUID,
 				Execution:        workflowExecution,
 				WorkflowTypeName: "visibility-workflow",
-				StartTimestamp:   startTime,
+				StartTime:        startTime,
 			},
 		})
 		s.Nil(err0)
@@ -780,7 +780,7 @@ func (s *VisibilityPersistenceSuite) TestDelete() {
 	}
 	nRows := 5
 	testNamespaceUUID := uuid.New()
-	startTime := time.Now().UTC().Add(time.Second * -5).UnixNano()
+	startTime := time.Now().UTC().Add(time.Second * -5)
 	for i := 0; i < nRows; i++ {
 		workflowExecution := commonpb.WorkflowExecution{
 			WorkflowId: uuid.New(),
@@ -791,7 +791,7 @@ func (s *VisibilityPersistenceSuite) TestDelete() {
 				NamespaceID:      testNamespaceUUID,
 				Execution:        workflowExecution,
 				WorkflowTypeName: "visibility-workflow",
-				StartTimestamp:   startTime,
+				StartTime:        startTime,
 			},
 		})
 		s.Nil(err0)
@@ -800,11 +800,11 @@ func (s *VisibilityPersistenceSuite) TestDelete() {
 				NamespaceID:      testNamespaceUUID,
 				Execution:        workflowExecution,
 				WorkflowTypeName: "visibility-workflow",
-				StartTimestamp:   startTime,
+				StartTime:        startTime,
 				Status:           enumspb.WORKFLOW_EXECUTION_STATUS_FAILED,
 			},
-			CloseTimestamp: time.Now().UnixNano(),
-			HistoryLength:  3,
+			CloseTime:     time.Now(),
+			HistoryLength: 3,
 		}
 		err1 := s.VisibilityMgr.RecordWorkflowExecutionClosed(closeReq)
 		s.Nil(err1)
@@ -813,7 +813,7 @@ func (s *VisibilityPersistenceSuite) TestDelete() {
 	resp, err3 := s.VisibilityMgr.ListClosedWorkflowExecutions(&p.ListWorkflowExecutionsRequest{
 		NamespaceID:       testNamespaceUUID,
 		EarliestStartTime: startTime,
-		LatestStartTime:   time.Now().UnixNano(),
+		LatestStartTime:   time.Now(),
 		PageSize:          10,
 	})
 	s.Nil(err3)
@@ -830,7 +830,7 @@ func (s *VisibilityPersistenceSuite) TestDelete() {
 		resp, err5 := s.VisibilityMgr.ListClosedWorkflowExecutions(&p.ListWorkflowExecutionsRequest{
 			NamespaceID:       testNamespaceUUID,
 			EarliestStartTime: startTime,
-			LatestStartTime:   time.Now().UnixNano(),
+			LatestStartTime:   time.Now(),
 			PageSize:          10,
 		})
 		s.Nil(err5)
@@ -847,14 +847,14 @@ func (s *VisibilityPersistenceSuite) TestUpsertWorkflowExecution() {
 		{
 			request: &p.UpsertWorkflowExecutionRequest{
 				VisibilityRequestBase: &p.VisibilityRequestBase{
-					NamespaceID:        "",
-					Namespace:          "",
-					Execution:          commonpb.WorkflowExecution{},
-					WorkflowTypeName:   "",
-					StartTimestamp:     0,
-					ExecutionTimestamp: 0,
-					TaskID:             0,
-					Memo:               nil,
+					NamespaceID:      "",
+					Namespace:        "",
+					Execution:        commonpb.WorkflowExecution{},
+					WorkflowTypeName: "",
+					StartTime:        time.Time{},
+					ExecutionTime:    time.Time{},
+					TaskID:           0,
+					Memo:             nil,
 					SearchAttributes: &commonpb.SearchAttributes{
 						IndexedFields: map[string]*commonpb.Payload{
 							searchattribute.TemporalChangeVersion: payload.EncodeBytes([]byte("dummy")),
@@ -868,16 +868,16 @@ func (s *VisibilityPersistenceSuite) TestUpsertWorkflowExecution() {
 		{
 			request: &p.UpsertWorkflowExecutionRequest{
 				VisibilityRequestBase: &p.VisibilityRequestBase{
-					NamespaceID:        "",
-					Namespace:          "",
-					Execution:          commonpb.WorkflowExecution{},
-					WorkflowTypeName:   "",
-					StartTimestamp:     0,
-					ExecutionTimestamp: 0,
-					TaskID:             0,
-					Memo:               nil,
-					SearchAttributes:   nil,
-					Status:             enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING,
+					NamespaceID:      "",
+					Namespace:        "",
+					Execution:        commonpb.WorkflowExecution{},
+					WorkflowTypeName: "",
+					StartTime:        time.Time{},
+					ExecutionTime:    time.Time{},
+					TaskID:           0,
+					Memo:             nil,
+					SearchAttributes: nil,
+					Status:           enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING,
 				},
 			},
 			// To avoid blocking the task queue processors on non-ElasticSearch visibility stores
@@ -897,8 +897,8 @@ func (s *VisibilityPersistenceSuite) assertClosedExecutionEquals(
 	s.Equal(req.Execution.RunId, resp.Execution.RunId)
 	s.Equal(req.Execution.WorkflowId, resp.Execution.WorkflowId)
 	s.Equal(req.WorkflowTypeName, resp.GetType().GetName())
-	s.Equal(s.nanosToMillis(req.StartTimestamp), s.nanosToMillis(timestamp.TimeValue(resp.GetStartTime()).UnixNano()))
-	s.Equal(s.nanosToMillis(req.CloseTimestamp), s.nanosToMillis(timestamp.TimeValue(resp.GetCloseTime()).UnixNano()))
+	s.Equal(p.UnixMilliseconds(req.StartTime), p.UnixMilliseconds(timestamp.TimeValue(resp.GetStartTime())))
+	s.Equal(p.UnixMilliseconds(req.CloseTime), p.UnixMilliseconds(timestamp.TimeValue(resp.GetCloseTime())))
 	s.Equal(req.Status, resp.GetStatus())
 	s.Equal(req.HistoryLength, resp.HistoryLength)
 }
@@ -908,12 +908,8 @@ func (s *VisibilityPersistenceSuite) assertOpenExecutionEquals(
 	s.Equal(req.Execution.GetRunId(), resp.Execution.GetRunId())
 	s.Equal(req.Execution.WorkflowId, resp.Execution.WorkflowId)
 	s.Equal(req.WorkflowTypeName, resp.GetType().GetName())
-	s.Equal(s.nanosToMillis(req.StartTimestamp), s.nanosToMillis(timestamp.TimeValue(resp.GetStartTime()).UnixNano()))
+	s.Equal(p.UnixMilliseconds(req.StartTime), p.UnixMilliseconds(timestamp.TimeValue(resp.GetStartTime())))
 	s.Nil(resp.CloseTime)
 	s.Equal(resp.Status, enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING)
 	s.Zero(resp.HistoryLength)
 }
-
-func (s *VisibilityPersistenceSuite) nanosToMillis(nanos int64) int64 {
-	return nanos / int64(time.Millisecond)
-}
