@@ -22,6 +22,8 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
+//go:generate mockgen -copyright_file ../../LICENSE -package mock -source $GOFILE -destination mock/store_mock.go -aux_files go.temporal.io/server/common/persistence=dataInterfaces.go
+
 package persistence
 
 import (
@@ -31,7 +33,6 @@ import (
 
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
-
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 )
 
@@ -45,14 +46,34 @@ type (
 	// ////////////////////////////////////////////////////////////////////
 	// Persistence interface is a lower layer of dataInterface.
 	// The intention is to let different persistence implementation(SQL,Cassandra/etc) share some common logic
-	// Right now the only common part is serialization/deserialization, and only ExecutionManager/HistoryManager need it.
-	// ShardManager/TaskManager/MetadataManager are the same.
+	// Right now the only common part is serialization/deserialization.
 	// ////////////////////////////////////////////////////////////////////
 
 	// ShardStore is a lower level of ShardManager
-	ShardStore = ShardManager
+	ShardStore interface {
+		Closeable
+		GetName() string
+		GetClusterName() string
+		CreateShard(request *InternalCreateShardRequest) error
+		GetShard(request *InternalGetShardRequest) (*InternalGetShardResponse, error)
+		UpdateShard(request *InternalUpdateShardRequest) error
+	}
+
 	// TaskStore is a lower level of TaskManager
-	TaskStore = TaskManager
+	TaskStore interface {
+		Closeable
+		GetName() string
+		CreateTaskQueue(request *InternalCreateTaskQueueRequest) error
+		GetTaskQueue(request *InternalGetTaskQueueRequest) (*InternalGetTaskQueueResponse, error)
+		ExtendLease(request *InternalExtendLeaseRequest) error
+		UpdateTaskQueue(request *InternalUpdateTaskQueueRequest) (*UpdateTaskQueueResponse, error)
+		ListTaskQueue(request *ListTaskQueueRequest) (*InternalListTaskQueueResponse, error)
+		DeleteTaskQueue(request *DeleteTaskQueueRequest) error
+		CreateTasks(request *InternalCreateTasksRequest) (*CreateTasksResponse, error)
+		GetTasks(request *GetTasksRequest) (*InternalGetTasksResponse, error)
+		CompleteTask(request *CompleteTaskRequest) error
+		CompleteTasksLessThan(request *CompleteTasksLessThanRequest) (int, error)
+	}
 	// MetadataStore is a lower level of MetadataManager
 	MetadataStore interface {
 		Closeable
@@ -93,7 +114,7 @@ type (
 		CreateWorkflowExecution(request *InternalCreateWorkflowExecutionRequest) (*CreateWorkflowExecutionResponse, error)
 		DeleteWorkflowExecution(request *DeleteWorkflowExecutionRequest) error
 		DeleteCurrentWorkflowExecution(request *DeleteCurrentWorkflowExecutionRequest) error
-		GetCurrentExecution(request *GetCurrentExecutionRequest) (*GetCurrentExecutionResponse, error)
+		GetCurrentExecution(request *GetCurrentExecutionRequest) (*InternalGetCurrentExecutionResponse, error)
 
 		// Scan related methods
 		ListConcreteExecutions(request *ListConcreteExecutionsRequest) (*InternalListConcreteExecutionsResponse, error)
@@ -145,13 +166,13 @@ type (
 		// ReadHistoryBranch returns history node data for a branch
 		ReadHistoryBranch(request *InternalReadHistoryBranchRequest) (*InternalReadHistoryBranchResponse, error)
 		// ForkHistoryBranch forks a new branch from a old branch
-		ForkHistoryBranch(request *InternalForkHistoryBranchRequest) (*InternalForkHistoryBranchResponse, error)
+		ForkHistoryBranch(request *InternalForkHistoryBranchRequest) error
 		// DeleteHistoryBranch removes a branch
 		DeleteHistoryBranch(request *InternalDeleteHistoryBranchRequest) error
 		// GetHistoryTree returns all branch information of a tree
-		GetHistoryTree(request *GetHistoryTreeRequest) (*GetHistoryTreeResponse, error)
+		GetHistoryTree(request *GetHistoryTreeRequest) (*InternalGetHistoryTreeResponse, error)
 		// GetAllHistoryTreeBranches returns all branches of all trees
-		GetAllHistoryTreeBranches(request *GetAllHistoryTreeBranchesRequest) (*GetAllHistoryTreeBranchesResponse, error)
+		GetAllHistoryTreeBranches(request *GetAllHistoryTreeBranchesRequest) (*InternalGetAllHistoryTreeBranchesResponse, error)
 	}
 
 	// VisibilityStore is the store interface for visibility
@@ -178,18 +199,19 @@ type (
 	// Queue is a store to enqueue and get messages
 	Queue interface {
 		Closeable
+		Init(blob *commonpb.DataBlob) error
 		EnqueueMessage(blob commonpb.DataBlob) error
 		ReadMessages(lastMessageID int64, maxCount int) ([]*QueueMessage, error)
 		DeleteMessagesBefore(messageID int64) error
-		UpdateAckLevel(messageID int64, clusterName string) error
-		GetAckLevels() (map[string]int64, error)
+		UpdateAckLevel(metadata *InternalQueueMetadata) error
+		GetAckLevels() (*InternalQueueMetadata, error)
 
 		EnqueueMessageToDLQ(blob commonpb.DataBlob) (int64, error)
 		ReadMessagesFromDLQ(firstMessageID int64, lastMessageID int64, pageSize int, pageToken []byte) ([]*QueueMessage, []byte, error)
 		DeleteMessageFromDLQ(messageID int64) error
 		RangeDeleteMessagesFromDLQ(firstMessageID int64, lastMessageID int64) error
-		UpdateDLQAckLevel(messageID int64, clusterName string) error
-		GetDLQAckLevels() (map[string]int64, error)
+		UpdateDLQAckLevel(metadata *InternalQueueMetadata) error
+		GetDLQAckLevels() (*InternalQueueMetadata, error)
 	}
 
 	// QueueMessage is the message that stores in the queue
@@ -200,6 +222,101 @@ type (
 		Encoding  string    `json:"message_encoding"`
 	}
 
+	InternalQueueMetadata struct {
+		Blob    *commonpb.DataBlob
+		Version int64
+	}
+
+	// InternalCreateShardRequest is used by ShardStore to create new shard
+	InternalCreateShardRequest struct {
+		ShardID   int32
+		RangeID   int64
+		ShardInfo *commonpb.DataBlob
+	}
+
+	// InternalGetShardRequest is used by ShardStore to retrieve a shard
+	InternalGetShardRequest struct {
+		ShardID int32
+	}
+
+	// InternalGetShardResponse is the response to GetShard
+	InternalGetShardResponse struct {
+		ShardInfo *commonpb.DataBlob
+	}
+
+	// InternalUpdateShardRequest is used by ShardStore to update a shard
+	InternalUpdateShardRequest struct {
+		ShardID         int32
+		RangeID         int64
+		ShardInfo       *commonpb.DataBlob
+		PreviousRangeID int64
+	}
+
+	InternalCreateTaskQueueRequest struct {
+		NamespaceID   string
+		TaskQueue     string
+		TaskType      enumspb.TaskQueueType
+		RangeID       int64
+		TaskQueueInfo *commonpb.DataBlob
+	}
+
+	InternalGetTaskQueueRequest struct {
+		NamespaceID string
+		TaskQueue   string
+		TaskType    enumspb.TaskQueueType
+	}
+
+	InternalGetTaskQueueResponse struct {
+		RangeID       int64
+		TaskQueueInfo *commonpb.DataBlob
+	}
+
+	InternalExtendLeaseRequest struct {
+		NamespaceID   string
+		TaskQueue     string
+		TaskType      enumspb.TaskQueueType
+		RangeID       int64
+		TaskQueueInfo *commonpb.DataBlob
+	}
+
+	InternalUpdateTaskQueueRequest struct {
+		NamespaceID   string
+		TaskQueue     string
+		TaskType      enumspb.TaskQueueType
+		TaskQueueKind enumspb.TaskQueueKind
+		RangeID       int64
+		ExpiryTime    *time.Time
+		TaskQueueInfo *commonpb.DataBlob
+	}
+
+	InternalCreateTasksRequest struct {
+		NamespaceID   string
+		TaskQueue     string
+		TaskType      enumspb.TaskQueueType
+		RangeID       int64
+		TaskQueueInfo *commonpb.DataBlob
+		Tasks         []*InternalCreateTask
+	}
+
+	InternalCreateTask struct {
+		TaskId     int64
+		ExpiryTime *time.Time
+		Task       *commonpb.DataBlob
+	}
+
+	InternalGetTasksResponse struct {
+		Tasks []*commonpb.DataBlob
+	}
+
+	InternalListTaskQueueResponse struct {
+		Items         []*InternalListTaskQueueItem
+		NextPageToken []byte
+	}
+	InternalListTaskQueueItem struct {
+		TaskQueue *commonpb.DataBlob // serialized PersistedTaskQueueInfo
+		RangeID   int64
+	}
+
 	// DataBlob represents a blob for any binary data.
 	// It contains raw data, and metadata(right now only encoding) in other field
 	// Note that it should be only used for Persistence layer, below dataInterface and application(historyEngine/etc)
@@ -218,19 +335,18 @@ type (
 
 	// InternalWorkflowMutableState indicates workflow related state for Persistence Interface
 	InternalWorkflowMutableState struct {
-		ActivityInfos       map[int64]*persistencespb.ActivityInfo
-		TimerInfos          map[string]*persistencespb.TimerInfo
-		ChildExecutionInfos map[int64]*persistencespb.ChildExecutionInfo
-		RequestCancelInfos  map[int64]*persistencespb.RequestCancelInfo
-		SignalInfos         map[int64]*persistencespb.SignalInfo
+		ActivityInfos       map[int64]*commonpb.DataBlob  // ActivityInfo
+		TimerInfos          map[string]*commonpb.DataBlob // TimerInfo
+		ChildExecutionInfos map[int64]*commonpb.DataBlob  // ChildExecutionInfo
+		RequestCancelInfos  map[int64]*commonpb.DataBlob  // RequestCancelInfo
+		SignalInfos         map[int64]*commonpb.DataBlob  // SignalInfo
 		SignalRequestedIDs  []string
-		ExecutionInfo       *persistencespb.WorkflowExecutionInfo
-		ExecutionState      *persistencespb.WorkflowExecutionState
+		ExecutionInfo       *commonpb.DataBlob // WorkflowExecutionInfo
+		ExecutionState      *commonpb.DataBlob // WorkflowExecutionState
 		NextEventID         int64
-
-		BufferedEvents  []*commonpb.DataBlob
-		Checksum        *persistencespb.Checksum
-		DBRecordVersion int64
+		BufferedEvents      []*commonpb.DataBlob
+		Checksum            *commonpb.DataBlob // persistencespb.Checksum
+		DBRecordVersion     int64
 	}
 
 	// InternalUpdateWorkflowExecutionRequest is used to update a workflow execution for Persistence Interface
@@ -262,21 +378,28 @@ type (
 
 	// InternalWorkflowMutation is used as generic workflow execution state mutation for Persistence Interface
 	InternalWorkflowMutation struct {
-		ExecutionInfo    *persistencespb.WorkflowExecutionInfo
-		ExecutionState   *persistencespb.WorkflowExecutionState
-		NextEventID      int64
-		LastWriteVersion int64
-		DBRecordVersion  int64
+		// TODO: properly set this on call sites
+		NamespaceID string
+		WorkflowID  string
+		RunID       string
 
-		UpsertActivityInfos       map[int64]*persistencespb.ActivityInfo
+		ExecutionInfo      *commonpb.DataBlob
+		ExecutionState     *persistencespb.WorkflowExecutionState
+		ExecutionStateBlob *commonpb.DataBlob
+		NextEventID        int64
+		StartVersion       int64
+		LastWriteVersion   int64
+		DBRecordVersion    int64
+
+		UpsertActivityInfos       map[int64]*commonpb.DataBlob
 		DeleteActivityInfos       map[int64]struct{}
-		UpsertTimerInfos          map[string]*persistencespb.TimerInfo
+		UpsertTimerInfos          map[string]*commonpb.DataBlob
 		DeleteTimerInfos          map[string]struct{}
-		UpsertChildExecutionInfos map[int64]*persistencespb.ChildExecutionInfo
+		UpsertChildExecutionInfos map[int64]*commonpb.DataBlob
 		DeleteChildExecutionInfos map[int64]struct{}
-		UpsertRequestCancelInfos  map[int64]*persistencespb.RequestCancelInfo
+		UpsertRequestCancelInfos  map[int64]*commonpb.DataBlob
 		DeleteRequestCancelInfos  map[int64]struct{}
-		UpsertSignalInfos         map[int64]*persistencespb.SignalInfo
+		UpsertSignalInfos         map[int64]*commonpb.DataBlob
 		DeleteSignalInfos         map[int64]struct{}
 		UpsertSignalRequestedIDs  map[string]struct{}
 		DeleteSignalRequestedIDs  map[string]struct{}
@@ -290,22 +413,29 @@ type (
 
 		Condition int64
 
-		Checksum *persistencespb.Checksum
+		Checksum *commonpb.DataBlob
 	}
 
 	// InternalWorkflowSnapshot is used as generic workflow execution state snapshot for Persistence Interface
 	InternalWorkflowSnapshot struct {
-		ExecutionInfo    *persistencespb.WorkflowExecutionInfo
-		ExecutionState   *persistencespb.WorkflowExecutionState
-		LastWriteVersion int64
-		NextEventID      int64
-		DBRecordVersion  int64
+		// TODO: properly set this on call sites
+		NamespaceID string
+		WorkflowID  string
+		RunID       string
 
-		ActivityInfos       map[int64]*persistencespb.ActivityInfo
-		TimerInfos          map[string]*persistencespb.TimerInfo
-		ChildExecutionInfos map[int64]*persistencespb.ChildExecutionInfo
-		RequestCancelInfos  map[int64]*persistencespb.RequestCancelInfo
-		SignalInfos         map[int64]*persistencespb.SignalInfo
+		ExecutionInfo      *commonpb.DataBlob
+		ExecutionState     *persistencespb.WorkflowExecutionState
+		ExecutionStateBlob *commonpb.DataBlob
+		StartVersion       int64
+		LastWriteVersion   int64
+		NextEventID        int64
+		DBRecordVersion    int64
+
+		ActivityInfos       map[int64]*commonpb.DataBlob
+		TimerInfos          map[string]*commonpb.DataBlob
+		ChildExecutionInfos map[int64]*commonpb.DataBlob
+		RequestCancelInfos  map[int64]*commonpb.DataBlob
+		SignalInfos         map[int64]*commonpb.DataBlob
 		SignalRequestedIDs  map[string]struct{}
 
 		TransferTasks    []Task
@@ -315,7 +445,13 @@ type (
 
 		Condition int64
 
-		Checksum *persistencespb.Checksum
+		Checksum *commonpb.DataBlob
+	}
+
+	InternalGetCurrentExecutionResponse struct {
+		RunID            string
+		ExecutionState   *persistencespb.WorkflowExecutionState
+		LastWriteVersion int64
 	}
 
 	// InternalHistoryNode represent a history node metadata
@@ -338,6 +474,8 @@ type (
 		Info string
 		// The branch to be appended
 		BranchInfo *persistencespb.HistoryBranch
+		// Serialized TreeInfo
+		TreeInfo *commonpb.DataBlob
 		// The history node
 		Node InternalHistoryNode
 		// Used in sharded data stores to identify which shard to use
@@ -360,6 +498,8 @@ type (
 	InternalForkHistoryBranchRequest struct {
 		// The base branch to fork from
 		ForkBranchInfo *persistencespb.HistoryBranch
+		// Serialized TreeInfo
+		TreeInfo *commonpb.DataBlob
 		// The nodeID to fork from, the new branch will start from ( inclusive ), the base branch will stop at(exclusive)
 		ForkNodeID int64
 		// branchID of the new branch
@@ -390,10 +530,18 @@ type (
 
 	// InternalDeleteHistoryBranchRequest is used to remove a history branch
 	InternalDeleteHistoryBranchRequest struct {
-		// branch to be deleted
-		BranchInfo *persistencespb.HistoryBranch
 		// Used in sharded data stores to identify which shard to use
-		ShardID int32
+		ShardID  int32
+		TreeId   string // TreeId, BranchId is used to delete target history branch itself.
+		BranchId string
+		// branch ranges is used to delete range of history nodes from target branch and it ancestors.
+		BranchRanges []InternalDeleteHistoryBranchRange
+	}
+
+	// InternalDeleteHistoryBranchRange is used to delete a range of history nodes of a branch
+	InternalDeleteHistoryBranchRange struct {
+		BranchId    string
+		BeginNodeId int64 // delete nodes with ID >= BeginNodeId
 	}
 
 	// InternalReadHistoryBranchRequest is used to read a history branch
@@ -434,19 +582,44 @@ type (
 		NextPageToken []byte
 	}
 
+	// InternalGetAllHistoryTreeBranchesResponse is response to GetAllHistoryTreeBranches
+	// Only used by persistence layer
+	InternalGetAllHistoryTreeBranchesResponse struct {
+		// pagination token
+		NextPageToken []byte
+		// all branches of all trees
+		Branches []InternalHistoryBranchDetail
+	}
+
+	// InternalHistoryBranchDetail used by InternalGetAllHistoryTreeBranchesResponse
+	InternalHistoryBranchDetail struct {
+		TreeID   string
+		BranchID string
+		Encoding string
+		Data     []byte // HistoryTreeInfo blob
+	}
+
+	// InternalGetHistoryTreeResponse is response to GetHistoryTree
+	// Only used by persistence layer
+	InternalGetHistoryTreeResponse struct {
+		// TreeInfos
+		TreeInfos []*commonpb.DataBlob
+	}
+
 	// VisibilityWorkflowExecutionInfo is visibility info for internal response
 	VisibilityWorkflowExecutionInfo struct {
-		WorkflowID       string
-		RunID            string
-		TypeName         string
-		StartTime        time.Time
-		ExecutionTime    time.Time
-		CloseTime        time.Time
-		Status           enumspb.WorkflowExecutionStatus
-		HistoryLength    int64
-		Memo             *commonpb.DataBlob
-		TaskQueue        string
-		SearchAttributes map[string]interface{}
+		WorkflowID           string
+		RunID                string
+		TypeName             string
+		StartTime            time.Time
+		ExecutionTime        time.Time
+		CloseTime            time.Time
+		Status               enumspb.WorkflowExecutionStatus
+		HistoryLength        int64
+		StateTransitionCount int64
+		Memo                 *commonpb.DataBlob
+		TaskQueue            string
+		SearchAttributes     map[string]interface{}
 	}
 
 	// InternalListWorkflowExecutionsResponse is response from ListWorkflowExecutions
@@ -464,18 +637,19 @@ type (
 
 	// InternalRecordWorkflowExecutionStartedRequest request to RecordWorkflowExecutionStarted
 	InternalVisibilityRequestBase struct {
-		NamespaceID        string
-		WorkflowID         string
-		RunID              string
-		WorkflowTypeName   string
-		StartTimestamp     int64
-		Status             enumspb.WorkflowExecutionStatus
-		ExecutionTimestamp int64
-		TaskID             int64
-		ShardID            int32
-		Memo               *commonpb.DataBlob
-		TaskQueue          string
-		SearchAttributes   *commonpb.SearchAttributes
+		NamespaceID          string
+		WorkflowID           string
+		RunID                string
+		WorkflowTypeName     string
+		StartTime            time.Time
+		Status               enumspb.WorkflowExecutionStatus
+		ExecutionTime        time.Time
+		StateTransitionCount int64
+		TaskID               int64
+		ShardID              int32
+		Memo                 *commonpb.DataBlob
+		TaskQueue            string
+		SearchAttributes     *commonpb.SearchAttributes
 	}
 
 	// InternalRecordWorkflowExecutionStartedRequest request to RecordWorkflowExecutionStarted
@@ -486,9 +660,9 @@ type (
 	// InternalRecordWorkflowExecutionClosedRequest is request to RecordWorkflowExecutionClosed
 	InternalRecordWorkflowExecutionClosedRequest struct {
 		*InternalVisibilityRequestBase
-		CloseTimestamp int64
-		HistoryLength  int64
-		Retention      *time.Duration
+		CloseTime     time.Time
+		HistoryLength int64
+		Retention     *time.Duration
 	}
 
 	// InternalUpsertWorkflowExecutionRequest is request to UpsertWorkflowExecution
@@ -572,8 +746,9 @@ func NewDataBlob(data []byte, encodingTypeStr string) *commonpb.DataBlob {
 	}
 
 	encodingType, ok := enumspb.EncodingType_value[encodingTypeStr]
-	if !ok || enumspb.EncodingType(encodingType) != enumspb.ENCODING_TYPE_PROTO3 {
-		panic(fmt.Sprintf("Invalid incoding: \"%v\"", encodingTypeStr))
+	if !ok || (enumspb.EncodingType(encodingType) != enumspb.ENCODING_TYPE_PROTO3 &&
+		enumspb.EncodingType(encodingType) != enumspb.ENCODING_TYPE_JSON) {
+		panic(fmt.Sprintf("Invalid encoding: \"%v\"", encodingTypeStr))
 	}
 
 	return &commonpb.DataBlob{
