@@ -28,10 +28,12 @@ import (
 	"fmt"
 	"net/http"
 
+	"go.temporal.io/server/client"
 	"go.temporal.io/server/common/authorization"
 	"go.temporal.io/server/common/config"
 	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/log"
+	persistenceClient "go.temporal.io/server/common/persistence/client"
 	"go.temporal.io/server/common/resolver"
 	"go.temporal.io/server/common/rpc/encryption"
 )
@@ -49,6 +51,7 @@ type (
 		blockingStart bool
 
 		logger                     log.Logger
+		namespaceLogger            log.Logger
 		authorizer                 authorization.Authorizer
 		tlsConfigProvider          encryption.TLSConfigProvider
 		claimMapper                authorization.ClaimMapper
@@ -57,6 +60,8 @@ type (
 		persistenceServiceResolver resolver.ServiceResolver
 		elasticseachHttpClient     *http.Client
 		dynamicConfigClient        dynamicconfig.Client
+		customDataStoreFactory     persistenceClient.AbstractDataStoreFactory
+		clientFactoryProvider      client.FactoryProvider
 	}
 )
 
